#include <avr/io.h>
#include <util/delay.h>
#include "uart.h"

#define RED_PIN   PD5
#define GREEN_PIN PD6
#define BLUE_PIN  PD7

void uart_init( void ) 
{
//  for different BAUD rate change the project settings, or uncomment 
//  following two lines:	
//	#undef  BAUD           // avoid compiler warning
//  #define BAUD 115200
	
   #include <util/setbaud.h>  // requires defined BAUD
   
   UBRR0H = UBRRH_VALUE;
   UBRR0L = UBRRL_VALUE;
   #if USE_2X                 // defined in setbaud.h 
    UCSR0A |= (1 << U2X0);
   #else
    UCSR0A &= ~(1 << U2X0);
   #endif


    UCSR0C = _BV(UCSZ01) | _BV(UCSZ00); /* 8-bit data */
    UCSR0B = _BV(RXEN0) | _BV(TXEN0);   /* Enable RX and TX */	
}


int uart_putc( char c, FILE *stream )
{
   if (c == '\n') 
      uart_putc('\r',stream);
   
   loop_until_bit_is_set(UCSR0A, UDRE0); /* Wait until data register empty. */
   UDR0 = c;
   return 0;
}


void uart_puts(const char *s)
{
  /* toto je vasa uloha */
}

char uart_getc(void) 
{
    loop_until_bit_is_set(UCSR0A, RXC0); /* Wait until data exists. */
    return UDR0;
}

void delay(int delay)      // vlastna funkcia pre dlhsie casy 
{
  for (int i=1; i<=delay; i++)
  _delay_ms(1);
}

// Funkcia na inicializciu pinov
void initRGBled() {
    // Nastavenie pinov ako OUTPUT
    DDRD |= (1 << RED_PIN) | (1 << GREEN_PIN) | (1 << BLUE_PIN);
}

// Funkcia na nastavenie farby
void setRGBled(uint8_t r, uint8_t g, uint8_t b) {
    // Nastavenie hodnt pre erven, zelen a modr LED didu
    if (r > 0) {
        PORTD |= (1 << RED_PIN);
    } else {
        PORTD &= ~(1 << RED_PIN);
    }

    if (g > 0) {
        PORTD |= (1 << GREEN_PIN);
    } else {
        PORTD &= ~(1 << GREEN_PIN);
    }

    if (b > 0) {
        PORTD |= (1 << BLUE_PIN);
    } else {
        PORTD &= ~(1 << BLUE_PIN);
    }
}